

die () {
  echo "submit.sh: error: $*" 1>&2
  exit 1
}

test $# = 1 || die "expected exactly one argument"
test "x$1" = x && die "invalid empty argument"

cp=cp
root="../.."

useaigunconstraint () {
  [ -d $root/bin ] || die "no '$root/bin/'"
  [ -f $root/bin/aigunconstraint ] || die "no '$root/bin/aigunconstraint'"
  cp="`( cd $root/bin/; pwd )`/aigunconstraint"
  [ -f $cp ] || \
    die "failed to get absolute path to '$root/bin/aigunconstraint'"
  echo "using '$cp' to copy and unconstraint AIGs"
}

modelchecker=$1
case $modelchecker in
  avr|btormc|cosa2|nuxmv|pono)
    subdir=btor2/bv
    ;;
  abc|abc17|avy|camical|nmtip|pdtrav)
    subdir=aig
    useaigunconstraint
    ;;
  *)
    die "invalid model checker '$modelchecker'"
    ;;
esac
[ -d $modelchecker ] && \
  die "directory '$modelchecker' exists (rename or delete it first)"
[ -d $root/modelcheckers ] || die "no '$root/modelcheckers'"
[ -d $root/modelcheckers/$modelchecker ] || \
  die "no '$root/modelcheckers/$modelchecker'"
[ -f $root/modelcheckers/$modelchecker/run.sh ] || \
  die "no '$root/modelcheckers/$modelchecker/run.sh'"
run="`( cd $root/modelcheckers/$modelchecker/; pwd )`/run.sh"
[ -f $run ] || \
  die "failed to get absolute path to '$root/modelcheckers/$modelchecker/run.sh'"
echo "using run script '$run'"

[ -d $root/bin ] || die "no '$root/bin/'"
[ -f $root/bin/runlim ] || die "no '$root/bin/'"
runlim="`( cd $root/bin; pwd )`"/runlim
[ -f $runlim ] || die "failed to get absolute path to '$root/bin/runlim'"
runlimversion="`$runlim --version`"
[ x"$runlimversion" = x ] && die "could not get version of '$runlim'"
echo "using '$runlim' version '$runlimversion'"

[ -d ../benchmarks ] || die "no '../benchmarks'"
[ -d ../benchmarks/$subdir ] || die "no '../benchmarks/$subdir'"
dir=`( cd ../benchmarks/$subdir; pwd )`
[ -d $dir ] || die "failed to get absolute path to '../benchmarks/$subdir'"
numbenchmarks=`find $dir -type f |wc -l`
test $numbenchmarks = 0 && die "no benchmarks found"
echo "found $numbenchmarks benchmarks in '$dir'"
mkdir $modelchecker || exit 1
list=$modelchecker/benchmarks
find $dir -type f | \
  xargs ls -Sr | \
  awk '{print NR, $0}' \
  > $modelchecker/benchmarks
count=`wc -l < $list`
test $count = $numbenchmarks || \
  die "generate benchmark list '$list' of size '$count'"
uniq=`sed -e 's,.*/,,' -e 's,\.[^\.]*$,,' $list|sort|uniq|wc -l`
test $uniq = $numbenchmarks || \
  die "different number $uniq of unique benchmark in '$list'"

sed \
  -e s,@MODELCHECKER@,$modelchecker, \
  -e s,@NUMBENCHMARKS@,$numbenchmarks, \
  -e s,@RUNLIM@,$runlim, \
  -e s,@RUN@,$run, \
  -e s,@CP@,$cp, \
  array.sh > $modelchecker/array.sh

cd $modelchecker
exec sbatch ./array.sh
